;+
;
; PROJECT:  CHIANTI
;
;       CHIANTI is an atomic database package for the calculation of
;       astrophysical emission line spectra.  It is a collaborative project
;       involving Ken Dere (Naval Research Laboratory, Washington DC), 
;       Brunella Monsignori-Fossi and Enrico Landi (Arcetri Observatory, 
;       Florence), and Helen Mason and Peter Young (DAMTP, Cambridge Univ.).
;
;
; NAME:
;	DESCALE2_UPS
;
; PURPOSE:
;
;	convert from Burgess-Tully scaling spline fits to Upsilons
;
; CATEGORY:
;
;	: science
;
; CALLING SEQUENCE:
;
;       DESCALE2_UPS,xt,c,t_type,spline_values,upsilon
;
;
; INPUTS:
;
;	xt:  scaled temperature
;        c:  Burgess-Tully scaling parameter
;       t_type:  B-T transition type (1-4)
;       spline_values:  spline values for B-T scaled upsilons	
;
; OPTIONAL INPUTS:
;
;	None:
;	
; KEYWORD PARAMETERS:
;
;	None
;
; OUTPUTS:
;
;	Upsilon:  the Maxwellian averaged collision strength
;
;
; OPTIONAL OUTPUTS:
;
;	None
;
; COMMON BLOCKS:
;
;	None:	
;
;
; PROCEDURE:
;
;	see Burgess and Tully, 1992, Astron and Astrophys, 254, 436.
;
; EXAMPLE:
;
;             would not be illuminating
;
; MODIFICATION HISTORY:
; 	Written by:	Ken Dere
;	March 1996:     Version 2.0
;
;-
pro descale2_ups,xt,c,t_type,spl,ups
;
;   to rescale the energies and upsilons
;   (thermal collision strengths)
;   modified version of descale_ups which uses data in common
;
;  xt=kt/de
;
;
;
CASE t_type OF

1:  begin
      st=1.-alog(c)/alog(xt+c)
      xs=findgen(5)*0.25
      y2=nr_spline(xs,spl)
      sups=nr_splint(xs,spl,y2,st)
      ups=sups*alog(xt+exp(1.))
    end
;
2:  begin
      st=xt/(xt+c)
      xs=findgen(5)*0.25
      y2=nr_spline(xs,spl)
      sups=nr_splint(xs,spl,y2,st)
      ups=sups
    end
;
3:  begin
      st=xt/(xt+c)
      xs=findgen(5)*0.25
      y2=nr_spline(xs,spl)
      sups=nr_splint(xs,spl,y2,st)
      ups=sups/(xt+1.)
;
    end
;
4:  begin
      st=1.-alog(c)/alog(xt+c)
      xs=findgen(5)*0.25
      y2=nr_spline(xs,spl)
      sups=nr_splint(xs,spl,y2,st)
      ups=sups*alog(xt+c)
   end
;
else:  print,' t_type ne 1,2,3,4=',t_type(indx,jndx),indx,jndx
;
ENDCASE
;
ups=ups>0.
;
if((max(st) gt 1.) or (min(st) lt 0.)) then print,xt,st, ' st outside 0. to 1. range'
return
end
;
