function fold_ioneq,emis,z,ion,logt=logt,tmax=tmax,trng=trng,level=level,$
	eqfile=eqfile,chifil=chifil,verbose=verbose, _extra=e
;+
;function	fold_ioneq
;	folds in ionization equilibrium fractions of the various ions
;	of a given element with line emissivities (which include only
;	level population info; see RD_LINE.PRO).  returns
;	emis(Z,ion,T)*ioneq(Z,ion,T)
;
;syntax
;	lflx=fold_ioneq(emis,Z,ion,logt=logt,tmax=tmax,trng=trng,$
;	level=level,chifil=chifil,chidir=chidir,eqfile=eqfile,verbose=v)
;
;parameters
;	emis	[INPUT; required] 1- or 2-D array of line emissivities as
;		a function of temperature
;		* if 2D, assumed to be EMIS(Temperature,Wavelength)
;	Z	[INPUT; required] atomic number(s)
;		* if size does not match the 2nd dimension of EMIS, then
;		  if >, ignore extra elements
;		  if <, but >1, use available ones, and use Z=1 for rest
;		  if 1, use same for all
;	ion	[INPUT; default: Z+1] ionic state
;		* NOTE: this is the ionic state >>that matters<<, i.e.,
;		  the ionic state of the species that populates the
;		  upper level.  See keyword JON of RD_LINE, which is
;		  what is needed here.
;
;keywords
;	logt	[INPUT] 1D array of log10(temperature [K]) at which EMIS
;		is given. (default: findgen(81)*0.05+4)
;		* if size does not match that of EMIS, use default
;	level	[INPUT; default: 0.5] level at which to determine the
;		temperature range on line flux (in other words, default
;		will be to return the FWHM)
;		* if <1, TRNG=LOGT(where(FLUX>LEVEL*MAX(FLUX))
;		* if >=1, TRNG=LOGT(where(FLUX>(LEVEL/100.)*MAX(FLUX))
;		* if <=0 or >=100, set to 0.5
;	tmax	[OUTPUT] log10(temperature) at which contribution is maximum
;	trng	[OUTPUT] range in LOGT (LEVEL of MAX)
;		-- TRNG(0,*) is lower bound, TRNG(1,*) is upper bound
;	eqfile	[I/O] file from which to read in ion-balance data
;		* default is to read in the CHIANTI file, !IONEQF
;		* hard-coded default is
;		  ioneq/mazzotta_etal.ioneq
;		* if not given, but CHIFIL is given, uses that.
;	chifil	[I/O] if set, reads in from CHIANTI-type database; must be
;		set to the name of the file containing the ionization
;		equilibrium info (i.e., CHIDIR/CHIFIL)
;		* may be overridden with EQFILE
;		* default is ioneq/mazzotta_etal.ioneq
;		* currently, set automatically if not given.  
;	verbose	[INPUT] higher the number, greater the chatter
;	_extra	[INPUT] use to transmit defined keywords to called functions
;		* RD_IONEQ [CHIDIR]
;
;subroutines
;	-- WHEE
;	-- SETSYSVAL
;	-- RD_IONEQ [READ_IONEQ (%)]
;	(%) CHIANTI subroutine, used as is
;
;history
;	vinay kashyap (Dec96)
;	removed call to KILROY and added call to WHEE, hacked default
;	  use of READ_IONEQ until more options become available; corrected
;	  bug that was reading in ionstate-1 rather than ionstate; added
;	  code to handle Z=0 (VK; Feb97)
;	bug: if Z is scalar take em all to be same element (VK; Apr97)
;	added keyword VERBOSE (VK; MarMM)
;	streamlined meanings of EQFILE, CHIFIL (VK; DecMM)
;	changed default for EQFILE; now pass VERBOSE to RD_IONEQ; added
;	  call to SETSYSVAL (VK; Jul01)
;	allowed EQFILE to be "none" (VK; Feb04)
;-

;	usage
if n_elements(emis) eq 0 or n_elements(z) eq 0 then begin
  print,'Usage: lflx=fold_ioneq(emissivity,Z,ion,logt=logt,tmax=tmax,trng=trng,$'
  print,'       level=level,eqfile=eqfile,chifil=chifil,chidir=chidir,verbose=v)'
  print,'  fold line emissivities read in with RD_LINE with ion balance'
  print,'  also accepts defined keywords CHIDIR and CHIFIL for RD_IONEQ'
  return,-1L
endif

;	check input
szl=size(emis) & nz=n_elements(z) & ni=n_elements(ion)
if nz eq 0 then begin & z=1 & nz=1 & endif	;default element is H
if ni eq 0 then begin & ion=z+1 & ni=nz & endif	;default ionic state is Z+1
case szl(0) of
  1: begin				;emis=emis(LOGT)
    nt=szl(1) & nw=1 & atno=[z(0)] & jon=[ion(0)]-1	;the "-1" coz of IDL
  end
  2: begin				;emis=emis(LOGT,WVL)
    nt=szl(1) & nw=szl(2) & atno=intarr(nw)+1 & jon=atno
    if nz le nw then atno(0:nz-1)=([z])(*) else atno(*)=z(0:nw-1)
    if ni le nw then jon(0:ni-1)=([ion])(*)-1 else jon(*)=ion(0:nw-1)-1
  end
  else: return,-1L			;nolle comprendo
endcase
;
if not keyword_set(logt) then begin
  tlog=findgen(nt)*(4./((nt>2)-1.))+4.
endif else tlog=logt
if n_elements(tlog) ne nt then begin
  message,'Line emissivity does not match temperature grid',/info
  tlog=findgen(nt)*(4./((nt>2)-1.))+4.
endif
;
if not keyword_set(level) then lev=0.5 else lev=level
if lev le 0. or lev ge 100. then lev=0.5
if lev ge 1. then lev=lev/100.
;
;	HACK ALERT!!!  until there are other ways to compute the
;	ion balance, use CHIANTI!!!  to avoid using CHIANTI, explicitly
;	set CHIFIL=0
if n_elements(chifil) eq 0 then chifil=1
;
zEQFILE='ioneq/mazzotta_etal.ioneq'
ivar=0 & defsysv,'!IONEQF',exists=ivar  ;if !IONEQF exists
if ivar ne 0 then setsysval,'IONEQF',zEQFILE,/getval
if not keyword_set(eqfile) then eqfile=zEQFILE
;def_eqfil='ioneq/arnaud_rothenflug.ioneq'
sze=size(eqfile) & nsze=n_elements(sze)
szc=size(chifil) & nszc=n_elements(szc)
if not keyword_set(eqfile) or sze(nsze-2) ne 7 then begin	;no EQFILE
  if szc(nszc-2) eq 7 then eqfile=chifil else eqfile=zEQFILE ;def_eqfil
endif

;	chatter
vv=0 & if keyword_set(verbose) then vv=long(verbose(0)) > 1

;	initialize
lflx=0*emis & tmax=fltarr(nw) & trng=fltarr(2,nw)

;	find all the unique elements
zz=atno(uniq(atno,sort(atno))) & mz=n_elements(zz)

;	for each element, read in ionization equilibrium and multiply
for iz=0,mz-1 do begin
  oz=where(atno eq zz(iz),noz)
  if zz(iz) le 0 then begin		;{error! error!
    ioneq=fltarr(nt)+1.
    if noz gt 0 then jon(oz)=0
  endif else begin			;}{all OK
    ioneq=fltarr(nt,zz(iz)+1)+1.	;default -- multiply by 1
    ;
    whee,spoke=spoke,/moveit	;we're working!
    if keyword_set(chifil) then begin	;use CHIANTI
      szc=size(chifil) & nszc=n_elements(szc)
      if szc(nszc-2) eq 7 then eqfile=chifil else begin
	if not keyword_set(eqfile) then eqfile=zEQFILE ;'ioneq/arnaud_rothenflug.ioneq'
      endelse
      ;eqfile=chifil
      ;if szc(nszc-2) ne 7 then eqfile=zEQFILE ;'ioneq/arnaud_rothenflug.ioneq'
      if strlowcase(eqfile) ne 'none' then begin
        if vv gt 0 then message,'Using ion balances from: '+eqfile,/info
        ioneq=rd_ioneq(zz(iz),tlog,eqfile=eqfile,verbose=vv,_extra=e)
      endif
    endif
  endelse				;}
  ;
  for ii=0,noz-1 do begin
    if vv gt 1 then whee,spoke=spoke,/moveit	;we're working!
    ;	fold in ionization equilibrium fractions
    lflx(*,oz(ii))=emis(*,oz(ii))*ioneq(*,jon(oz(ii)))
    ;	and here figure out TMAX ...
    tmp=reform(lflx(*,oz(ii))) ;& imax=where(tmp eq max(tmp),moo)
    tmpmax=max(tmp,imax)
    tmax(oz(ii))=total(tlog(imax));/moo
    ;	... and TRNG
    oo=where(tmp gt lev*tmpmax,moo)
    ;oo=where(tmp gt lev*tmp(imax(0)),moo)
    if moo ne 0 then begin
      trng(0,oz(ii))=tlog(oo(0)) & trng(1,oz(ii))=tlog(oo(moo-1))
    endif
  endfor
  if vv gt 1 then whee,spoke=spoke,/moveit	;we're working!
  ;
endfor

return,lflx
end
