function likeli,data,model,dsigma=dsigma,ulim=ulim,binom=binom,chi2=chi2,$
	rchi=rchi,cash=cash,castor=castor, _extra=e
;+
;function	likeli
;	returns p(D|M), the likelihood of observing the data for the
;	given model.
;
;	the default action is to return exp(-chi^2/2)
;
;syntax
;	prob=likeli(data,model,dsigma=dsigma,ulim=ulim,/binom,/chi2,$
;	/rchi,/cash,/castor)
;
;parameters
;	data	[INPUT; required] observed data
;	model	[INPUT; required] predicted model or model parameters
;
;keywords
;	dsigma	[INPUT] errors on DATA
;		* if not specified, taken to be 1+sqrt(abs(DATA)+0.75)
;	ulim	[INPUT] long-integer array specifying which elements of
;		DATA are upper limits (1: UL, 0: not)
;	cash	[INPUT] if set, returns the Cash (19??, ApJ 228, 939) statistic
;	castor	[INPUT] if set, returns the Castor correction to Cash
;	binom	[INPUT] if set, likelihood is computed as
;		    MODEL(0)^(d1)*(1.-MODEL(0))^(d0)
;		where
;		    d1=total(DATA(where(DATA ge BINOM))) and
;		    d0=total(DATA(where(DATA lt BINOM)))
;	chi2	[INPUT; default] if set, returns alog(p(D|M))
;	rchi	[INPUT] if set, returns alog(p(D|M))/n_elements(D)
;		* the priority is:
;		  CASH, CASTOR, BINOM, CHI2, RCHI
;	_extra	[INPUT] junk -- here only to avoid crashing the program
;
;history
;	vinay kashyap (Mar 97)
;	added keywords CASH and CASTOR (VK; Jul98)
;	changed keyword name SIGMA to DSIGMA (VK; MMaug)
;	had forgotten to actually include CASH and CASTOR in call (VK; May02)
;-

;	usage
nd=n_elements(data) & nm=n_elements(model)
if nd eq 0 or nm eq 0 then begin
  print,'Usage: prob=likeli(data,model,dsigma=dsigma,ulim=ulim,$'
  print,'    /cash,/castor,/binom,/chi2,/rchi)'
  print,'  returns p(D|M) or appropriate statistic'
  return,0.
endif

;	cast data
x=[data] & m=[model] & s=1.+sqrt(abs(x)+0.75) & u=lonarr(nd)
ns=n_elements(dsigma) & nu=n_elements(ulim)
if ns gt 0 then begin
  if ns le nd then s(0)=dsigma else s(*)=dsigma(0:nd-1)
endif
if nu gt 0 then begin
  if nu le nd then u(0)=ulim else u(*)=ulim(0:nd-1)
endif
if nd ne nm then begin
  message,'DATA and MODEL do not match',/info & return,0.
endif

;	if Cash...
if n_elements(cash) gt 0 then begin
  prob=0.
  oo=where(x ge 0 and m gt 0,moo)
  if moo lt nd then begin
    message,'warning.. some bins appear to have -ve D or M',/info
  endif
  if moo eq 0 then begin
    message,'no valid data points for Cash statistic -- returning 0',/info
  endif else prob=2*total(m(oo)-x(oo)*alog(m(oo))+lngamma(x(oo)+1.))
  return, prob
endif

;	if Castor...
if n_elements(castor) gt 0 then begin
  prob=0.
  oo=where(x gt 0 and m gt 0,moo)
  if moo lt nd then begin
    message,'warning.. some bins appear to have -ve D or M',/info
  endif
  if moo eq 0 then begin
    message,'no valid data points for Cash-Castor -- returning 0',/info
  endif else prob=2*total(m(oo)-x(oo)+x(oo)*(alog(x(oo))-alog(m(oo))))
  return, prob
endif

;	if binomial...
if n_elements(binom) gt 0 then begin
  oo=where(x lt binom(0),moo)
  if moo gt 0 then d0=total(x(oo)) else d0=0.
  oo=where(x ge binom(0),moo)
  if moo gt 0 then d1=total(x(oo)) else d1=0.
  theta=abs(model(0))<1
  prob=((theta^(d1) * (1.-theta)^(d0)) < 1)>1e-30
  if keyword_set(chi2) then prob=alog(prob)
  return,prob
endif

;	default ... exp(-chi^2/2)
oo=where(s le 0,moo) & if moo gt 0 then begin
  message,'replacing 0 and -ve errors by 1+sqrt(abs(DATA)+0.75)',/info
  s(oo)=1.+sqrt(abs(x(oo))+0.75)
endif
;	note: where u.NE.0.AND.M.lE.X, p(D|M)=1
oo=where(u ne 0 and m gt x,moo)
if moo gt 0 then begin			;p(D|M)=0: model violates upper limit
  if keyword_set(chi2) then return,2d30 else return,1d-30
endif
oo=where(u eq 0 and s gt 0,moo) & if moo gt 0 then begin
  chi22=total(((x(oo)-m(oo))/s(oo))^2)/2.
  if keyword_set(rchi) then chi22=chi22/moo	;reduced chi^2
  if not keyword_set(chi2) then begin
    chi22=chi22 < 69
    prob=exp(-chi22) < 1
  endif else prob=chi22
endif

return,prob
end
