pro conv_rmf,nrg,flx,chan,spec,rmf,effar=effar,nrgar=nrgar,rmfstr=rmfstr,$
	_extra=e
;+
;procedure	conv_rmf
;	convolves input energy spectrum with the response matrix
;
;	unlike the combination of RDRESP and FOLD_RESP, this one goes
;	easy on memory at the expense of speed.
;
;syntax
;	conv_rmf,nrg,flx,chan,spec,rmf,effar=effar,nrgar=nrgar,$
;	rmfstr=rmfstr,fchcol=fchcol,/shift1
;
;parameters
;	nrg	[INPUT; required] mid-bin values at which input FLX is
;		defined.
;		* usually [keV], depends on RMF file
;		* if size = N(FLX)+1, assumed to be bin-boundaries
;	flx	[INPUT; required] fluxes in [ph/bin/(s/cm^2/sr/...)]
;	chan	[OUTPUT; required] bin boundary values at which output SPEC
;		is defined
;		* same units as NRG
;	spec	[OUTPUT; required] output, convolved spectrum
;	rmf	[INPUT; required] name of OGIP-compliant FITS file containing
;		the response matrix
;		* may also be a structure, such as the output of RD_OGIP_RMF()
;
;keywords
;	effar	[INPUT] effective areas [cm^2]
;	nrgar	[INPUT] energies at which EFFAR are defined
;		* same units as NRG
;		* sizes of EFFAR and NRGAR must match
;		* range of NRGAR must overlap NRG
;		* if EFFAR and NRGAR are legal, FLX is multiplied by
;		  EFFAR (over intersection of NRGAR and NRG; zeroed
;		  elsewhere) before convolving with response matrix
;		* of course, this multiplication makes sense only if
;		  units on FLX are [ph/bin/cm^2/...]
;	rmfstr	[OUTPUT] structure containing info of response matrix,
;		the output of RD_OGIP_RMF()
;	_extra	[INPUT ONLY] pass defined keywords to subroutines:
;		RD_OGIP_RMF: FCHCOL, SHIFT1
;
;restrictions
;	requires IDLAstro library
;	requires subroutine RD_OGIP_RMF
;
;history
;	vinay kashyap (Apr99)
;	added keyword SHIFT1 (VK; JanMMI)
;	bug: incorrect usage of WVLAR instead of NRGAR in effar block
;	  (Erica R; Aug01)
;	deleted keyword SHIFT1, cleaned up and speeded up (VK; Nov2001)
;	bug: when spectrum size < RMF size and RMF has multiple groups,
;	  was using only the first group (VK; Nov'02)
;	bug: was crashing when NRG was outside the bounds of the RMF
;	  in cases where the RMF was at a higher resolution (LL/VK; Apr'03)
;-

;	usage
ok='ok'
np=n_params() & mnrg=n_elements(nrg) & mflx=n_elements(flx)
szr=size(rmf) & nszr=n_elements(szr)
nrmf=n_elements(rmf) & mrmf=n_tags(rmf)
if np lt 5 then ok='missing parameters' else $
 if mnrg eq 0 then ok='NRG is undefined: require spectrum grid' else $
  if mflx eq 0 then ok='FLX is undefined: require spectrum' else $
   if mnrg ne mflx and mnrg ne (mflx+1L) then ok='input spectrum garbled' else $
    if nrmf eq 0 then ok='RMF is undefined' else $
     if szr[nszr-2] ne 7 and mrmf eq 0 then $
     ok='RMF must be either a filename or a response matrix structure '
if ok ne 'ok' then begin
  print,'Usage: conv_rmf,nrg,flx,chan,spec,rmf,effar=effar,nrgar=nrgar,$'
  print,'       rmfstr=rmfstr,fchcol=fchcol,/shift1'
  print,'  convolve spectrum with response'
  if np gt 0 then message,ok,/info
  return
endif

;	check inputs
ee=[nrg[*]] & fe=[flx[*]]
if mnrg eq mflx+1 then begin
  ;	assume that NRG are bin boundaries and obtain the mid-bin values
  ee=[0.5*(nrg[1:*]+nrg)]
  mnrg=mnrg-1L
  if mnrg ne mflx then message,'BUG!'
endif
;if mnrg eq mflx then begin
;  ;	assume input NRG are mid-bin values and decipher the bin boundaries
;  de=nrg[1:*]-nrg
;  ee=[nrg-0.5*de,nrg[mnrg-1]+0.5*de[mnrg-2]*[-1,1]]
;  mnrg=mnrg+1L
;  if mnrg ne mflx+1L then message,'BUG!'
;endif

;	include effective areas?
mnar=n_elements(nrgar) & mear=n_elements(effar)
if mnar eq mear and mear gt 0 then begin	;(effective area defined
  arnrg=ee & areff=0.*arnrg+1.
  if mnar eq 1 then areff[*]=effar[0] else begin
    minea=min(effar,max=maxea) & minna=min(nrgar,max=maxna)
    areff=(interpol(effar,nrgar,ee) > (minea)) < (maxea)
    oo=where(arnrg lt minna or arnrg gt maxna,moo)
    if moo gt 0 then areff[oo]=0.
  endelse
  fe=fe*areff	;assumes input FLX is in units of [ph/bin/cm^2/...]
endif						;EFFAR(NRGAR))

;	read in response matrix
if mrmf eq 0 then rmfstr=rd_ogip_rmf(rmf[0]) else rmfstr=rmf
if n_tags(rmfstr) eq 0 then return
;
NNRG=rmfstr.NNRG & ELO=rmfstr.ELO & EHI=rmfstr.EHI
N_GRP=rmfstr.N_GRP & F_CHAN=rmfstr.F_CHAN & N_CHAN=rmfstr.N_CHAN & FIRSTCHAN=rmfstr.FIRSTCHAN
MATRIX=rmfstr.MATRIX
NCHAN=rmfstr.NCHAN & EMN=rmfstr.EMN & EMX=rmfstr.EMX
szn=size(N_CHAN)

;	define outputs
chan=[EMN,EMX[nchan-1L]] & spec=fltarr(nchan)

;	find binning for input NRG
;ie=binersp(ee,bstr=rmfstr)
ie=binersp(ee,elo,ehi)

if keyword_set(FIRSTCHAN) then message,$
	'assuming that input RMF is 1-based, not 0-based',/info

;	distribute each input FLX into PHAs
if mflx gt nnrg then begin	;(spectrum at higher resolution than RMF
  for inrg=0L,nnrg-1L do begin
    rsp=fltarr(nchan)
    ngrp=n_grp[inrg] & jbeg=0
    for ig=0L,ngrp-1L do begin
      if szn[0] gt 1 then begin
	ibeg=f_chan[ig,inrg] & iw=n_chan[ig,inrg]
      endif else begin
	ibeg=f_chan[inrg] & iw=n_chan[inrg]
      endelse
      if keyword_set(firstchan) then ibeg=ibeg-1	;IDL index correction
      if iw gt 0 then rsp[ibeg:ibeg+iw-1L]=matrix[jbeg:jbeg+iw-1L,inrg]
      jbeg=jbeg+iw
    endfor
    ;
    oi=where(ie eq inrg,moi)
    if moi gt 0 then spec=spec+rsp*total(fe[oi])
  endfor
endif else begin		;MFLX.GE.NNRG)(RMF at higher resolution than spectrum
  oi=where(ie ge 0,moi)
  for i=0L,moi-1L do begin
    ii=oi[i]
    inrg=ie[ii] & rsp=fltarr(nchan)
    ngrp=n_grp[inrg] & jbeg=0
    for ig=0L,ngrp-1L do begin
      if szn[0] gt 1 then begin
	ibeg=f_chan[ig,inrg] & iw=n_chan[ig,inrg]
      endif else begin
	ibeg=f_chan[inrg] & iw=n_chan[inrg]
      endelse
      if keyword_set(firstchan) then ibeg=ibeg-1	;IDL index correction
      if iw gt 0 then rsp[ibeg:ibeg+iw-1L]=matrix[jbeg:jbeg+iw-1L,inrg]
      jbeg=jbeg+iw
    endfor
    ;
    spec=spec+fe[ii]*rsp
  endfor
endelse				;NNRG>MFLX)

return
end
