pro poaintsym, symb, psize=psize,pthick=pthick,pfill=pfill,$
	npoint=npoint,verbose=verbose, _extra=e
;+
;procedure	poaintsym
;	Define useful plotting symbols not in the standard !PSYM
;	definitions.  After symbol has been defined with this
;	routine, a plotting command should follow with either
;	PSYM = 8 or !P.PSYM = 8 (see USERSYM).
;
;syntax
;	poaintsym,symb,psize=psize,pthick=pthick,/pfill,color=color,$
;	npoint=npoint,verbose=verbose
;
;parameters
;	symb	[INPUT; required] string describing the type of
;		symbol to define.  The following are accepted:
;		  'circle',
;		  'star',
;		  'square', 'diamond', 'box',
;		  'triangle', 'down triangle', 'right triangle', 'left triangle',
;		  'down arrow', 'up arrow', 'left arrow', 'right arrow'
;		  'tilde ANGLE OVERFLOW'
;		* if something unknown is specified, the program
;		  prints a list of accepted options and quits
;		  without making a change
;		* case TILDE: set ANGLE (default=0) to rotate the symbol,
;		  and OVERFLOW (default=1) to pull the ends out a bit
;
;keywords
;	psize	[INPUT] size of the plotting symbol in
;		multiples of the standard size (see SYMSIZE); does
;		not need to be an integer
;	pthick	[INPUT] thickness of symbol being drawn
;	pfill	[INPUT] if set, fills the symbol
;		* does not affect arrows or character symbols
;	npoint	[INPUT] number of vertices in the symbol
;		* note that usersym maximum is 49
;		* currently, only matters to TILDE
;	verbose	[INPUT] controls chatter
;	_extra	[INPUT ONLY] pass defined keywords to subroutines
;		USERSYM: COLOR
;
;method
;	Appropiate X,Y vectors are used to define the symbol and passed
;	to USERSYM
;
;examples
;	Plot with filled stars as the symbol, twice the default size
;	  POAINTSYM,'star',psize=2,/pfill & PLOT,findgen(10),PSYM=8
;	Plot with open circle as the symbol
;	  POAINTSYM,'circle' & PLOT,findgen(10),PSYM=8
;	Plot filled color circles
;	  POAINTSYM,'circle',/pfil,psize=3,col=2 & PLOT,findgen(10),PSYM=8
;	Plot arrows all around
;	  POAINTSYM,'left arrow',pthick=2,psize=2 & plot,findgen(11),psym=8
;	  POAINTSYM,'up arrow',pthick=3,psize=3 & oplot,findgen(11),psym=8
;	  POAINTSYM,'right arrow',pthick=4,psize=4 & oplot,findgen(11),psym=8
;	  POAINTSYM,'down arrow',pthick=5,psize=5 & oplot,findgen(11),psym=8
;	Plot tildes
;	  POAINTSYM,'tilde',pthick=5,psize=5 & plot,findgen(11),psym=8
;	  POAINTSYM,'tilda 90',pthick=5,psize=5 & plot,findgen(11),psym=8
;	  POAINTSYM,'tilda 90 3',pthick=5,psize=5 & plot,findgen(11),psym=8
;	  POAINTSYM,'tilda 0 20',pthick=2,psize=5,npoint=49 & plot,findgen(11),psym=8
;
;history
;	modified from JDSYM (Jeremy Drake 2001; JJD added square and
;	  diamond symbols to PLOTSYM [ASTROLib; Wayne Landsman 1992])
;	  by Vinay Kashyap (Oct03)
;	added option TILDE and keyword NPOINT (VK; Feb04)
;-

On_error,2

;	usage
ok='ok' & np=n_params() & ns=n_elements(symb) & ss=size(symb,/type)
if np eq 0 then ok='Insufficient parameters' else $
 if ns eq 0 then ok='SYMB: undefined' else $
  if ns gt 1 then ok='SYMB cannot be a vector' else $
   if ss ne 7 then ok='SYMB must be a character string'
if ok ne 'ok' then begin
  print,'Usage: poaintsym,symb,psize=psize,pthick=pthick,/pfill,verbose=verbose'
  print,'  define useful plotting symbols not in standard !PSYM definitions'
  if np ne 0 then message,ok,/informational
  return
endif

;	figure out keywords
psiz=1.0 & if keyword_set(psize) then psiz=psize[0]
pthk=1.0 & if keyword_set(pthick) then pthk=pthick[0]
pfil=0 & if keyword_set(pfill) then pfil=1
npt=15L & if keyword_set(npoint) then npt=long(npoint[0])
if npt lt 2 or npt gt 49 then begin
  message,'Number of points must be between 2 and MAX<50',/informational
  if npt lt 2 then npt=2L
  if npt ge 50 then npt=49L
endif
vv=0 & if keyword_set(verbose) then vv=long(verbose[0])>1

;	figure out what SYMB contains
ss=strtrim(strlowcase(symb[0]),2) & s1=strmid(ss,0,1) & csymb=ss
if s1 eq 'c' then begin				;(C*
  if strpos(ss,'c',0) ge 0 then csymb='CIRCLE'	;expand to CIR.. if other shapes are coded
endif						;C*)
if s1 eq 's' then begin				;(S*
  if strpos(ss,'st',0) ge 0 then csymb='STAR'
  if strpos(ss,'sq',0) ge 0 then csymb='SQUARE'
endif						;S*)
if s1 eq 'b' then begin				;(B*
  if strpos(ss,'b',0) ge 0 then csymb='SQUARE'
endif						;B*)
if s1 eq 'd' then begin				;(D*
  if strpos(ss,'di',0) ge 0 then csymb='DIAMOND'
endif						;D*)
if strpos(ss,'tr',0) ge 0 then begin		;(TR*
  csymb='UP TRIANGLE'
  if strpos(ss,'do',0) ge 0 then csymb='DOWN TRIANGLE'
  if strpos(ss,'rig',0) ge 0 then csymb='RIGHT TRIANGLE'
  if strpos(ss,'lef',0) ge 0 then csymb='LEFT TRIANGLE'
endif						;TRIANGLE)
if strpos(ss,'arr',0) ge 0 then begin		;(ARR*
  if strpos(ss,'do',0) ge 0 then csymb='DOWN ARROW'
  if strpos(ss,'up',0) ge 0 then csymb='UP ARROW'
  if strpos(ss,'rig',0) ge 0 then csymb='RIGHT ARROW'
  if strpos(ss,'lef',0) ge 0 then csymb='LEFT ARROW'
endif						;ARROW)
if strpos(ss,'lim',0) ge 0 then begin		;(LIM*
  if strpos(ss,'upp',0) ge 0 then csymb='DOWN ARROW'	;upper limit == down arrow
  if strpos(ss,'low',0) ge 0 then csymb='UP ARROW'	;lower limit == up arrow
endif						;LIMIT==ARROW)
if strpos(ss,'til',0) ge 0 then begin		;(TIL*
  csymb='TILDE'
  cc=strsplit(ss,'[ '+STRING(9B)+']+',/extract) & ncc=n_elements(cc)
  tilde_ang=0. & if ncc gt 1 then tilde_ang=float(cc[1])
  tilde_ovr=1. & if ncc gt 2 then tilde_ovr=float(cc[2])
  if vv gt 100 then stop
endif						;TILDE)
if vv gt 0 then message,'Setting point symbol to: '+csymb,/informational

case csymb of					;{CSYMB
  'CIRCLE': begin
    ang = 2*!PI*findgen(49)/48.     ;Get position every 5 deg
    xarr = psiz*cos(ang)  &  yarr = psiz*sin(ang)
  end
  'STAR': begin
    r = psiz   
    ang = (720. / 5*findgen(6) + 45) / !RADEG  ;Define star angles every 144 deg
    xarr = r*cos(ang)  & yarr = r*sin(ang)
  end
  'SQUARE': begin
    xarr = [1,-1,-1,1,1]*psiz
    yarr = [1,1,-1,-1,1]*psiz
  end
  'DIAMOND': begin
    xarr = [0,1.41,0,-1.41,0]*psiz
    yarr = [1.41,0,-1.41,0,1.41]*psiz
  end
  'UP TRIANGLE': begin
    xarr = [-1,0,1,-1]*psiz
    yarr = [-1,1,-1,-1]*psiz
  end
  'DOWN TRIANGLE': begin
    xarr = [-1, 0, 1, -1]*psiz
    yarr = [ 1,-1, 1, 1]*psiz
  end
  'RIGHT TRIANGLE': begin
    xarr = [-1,1,-1,-1]*psiz
    yarr = [1,0,-1,1]*psiz
  end
  'LEFT TRIANGLE': begin
    xarr = [1,1,-1,1]*psiz
    yarr = [1,-1,0,1]*psiz
  end
  'DOWN ARROW': begin
    xarr = [0,0,.5,0,-.5]*psiz
    yarr = [0,-2,-1.4,-2,-1.4]*psiz
    pfil = 0
  end
  'UP ARROW': begin
    xarr = [0,0,.5,0,-.5]*psiz
    yarr = [0,2,1.4,2,1.4]*psiz
    pfil = 0
  end
  'LEFT ARROW': begin
    yarr = [0, 0, 0.5, 0, -.5]*psiz
    xarr = [0,-2,-1.4,-2,-1.4]*psiz
    pfil = 0
  end
  'RIGHT ARROW': begin
    yarr = [ 0, 0, 0.5, 0, -.5] * psiz
    xarr = [ 0, 2, 1.4, 2, 1.4] * psiz
    pfil = 0
  end
  'TILDE': begin
    narr=npt/2L & marr=(2L*narr+1L < 49 ) > 2
    xarr = (findgen(marr)-marr/2)/(marr/2)
    if narr ne tilde_ovr then xarr = (findgen(marr)-narr)/(narr-tilde_ovr)
    yarr = (-sin(xarr*!pi))
    if tilde_ang ne 0 then begin
      xarr2 = xarr*cos(tilde_ang*!pi/180.) + yarr*sin(tilde_ang*!pi/180.)
      yarr2 = -xarr*sin(tilde_ang*!pi/180.) + yarr*cos(tilde_ang*!pi/180.)
      xarr = xarr2 & yarr=yarr2
    endif
    xarr = xarr * psiz
    yarr = yarr * psiz
    if vv gt 100 then stop
  end
  else: begin
    message,CSYMB+': plotting symbol not supported',/informational
    print,'Available options are:'
    print,'Circle'
    print,'STar'
    print,'SQuare / Diamond / Box'
    print,'up TRIangle / DOWn TRIangle / RIGht TRIangle /LEFt TRIangle'
    print,'UP ARRow / DOWn ARRow / LEFt ARRow / RIGht ARRow'
    print,'TILde <angle> <overshoot> <number of points>'
    return
  end
endcase						;CSYMB}

usersym, xarr, yarr, FILL = pfil, THICK=pthk, _extra=e

return
end          
